from abc import ABC, abstractmethod
from typing import TYPE_CHECKING, Dict, Optional, Type

if TYPE_CHECKING:
    from graphlin.node import Node


class AbstractRegistry(ABC):
    @abstractmethod
    def add(self, node):
        pass

    @abstractmethod
    def remove(self, node):
        pass

    @abstractmethod
    def get(self, label: str) -> Optional[Type["Node"]]:
        pass


class Registry(AbstractRegistry):
    _registry: Dict[str, list[Type["Node"]]]

    def __init__(self):
        self._registry = {}

    def add(self, node_cls: Type["Node"]):
        for label in node_cls.__full_labels__():
            if label not in self._registry:
                self._registry[label] = []
            self._registry[label].append(node_cls)

    def remove(self, node: "Node"):
        for label in node.full_labels:
            self._registry.pop(label)

    def get(self, label: str) -> Optional[list[Type["Node"]]]:
        return self._registry.get(label)


default_registry = Registry()
